"""
Computes posterior inferences related to post-intervention period of a time series
based model.
"""


from __future__ import absolute_import, division, print_function

import numpy as np
import pandas as pd
from dateutil.relativedelta import relativedelta

from causaltest.bin.conf import DATE_NAME, COEF_IMPACT

from causalimpact import CausalImpact


class Infer(object):
    """
    All computations related to the inference process of the post-intervention
    prediction is handled through the methods implemented here.
    """
    def _infer_pre_start_date(self):
        # TODO: find smarter way of computing pre_test_period
        try:
            print(self.pre_start_date)
        except AttributeError:
            self.pre_start_date = self.start_date + relativedelta(months=-1)

    def infer_end_date(self, baseline_budget, control_budget,
                       treatment_budget, start_date, coef=COEF_IMPACT):
        # start_date is a parameter in case it had to changed for pre-test definition
        budget_impact = ((treatment_budget - control_budget)/baseline_budget)*coef
        width = ((self.ci.summary_data.loc['abs_effect_upper', 'average']\
                   -self.ci.summary_data.loc['abs_effect_lower', 'average'])\
                   /2)/self.ci.summary_data.loc['actual', 'average']
        days = np.log(1+width) / np.log(1+budget_impact)
        days = int(days) + 7 # add a week for confort
        self.end_date = start_date + relativedelta(days=days)
        return self.end_date

    def _process_fact_geo(self):
        """
        Wrangle data to fit causalimpact library.
        """
        self.data_ci = self.fact_geo.copy()
        self.data_ci = self.data_ci.loc[~self.data_ci.id.isin(self.exclude_geo_id)]
        self.data_ci['group_'] = self.data_ci.id.map(lambda x: 'treatment' \
                                                     if x in self.treatment_geo_id \
                                                     else 'control')
        self.data_ci = self.data_ci.drop('id', axis=1)\
                                   .groupby(['group_', DATE_NAME])\
                                   .sum()\
                                   .unstack(level=0)

        self.data_ci.columns = self.data_ci.columns.get_level_values(1)
        dates_index = self.data_ci.reset_index()[DATE_NAME]
        self.data_ci = self.data_ci.reset_index()\
                                  .drop(DATE_NAME, axis=1)\
                                  .rename(columns={'treatment':'y',
                                                   'control':'X'})\
                                  [['y', 'X']]
                                  
        self.pre_period = [int(dates_index[dates_index == self.pre_start_date].index.values[0]),
                      int(dates_index[dates_index == self.start_date].index.values[0])-1]
        self.post_period = [int(dates_index[dates_index == self.start_date].index.values[0]), len(dates_index)-1]
        #return self._inferences

    def _infer_causal_impact(self):
        """

        """
        self._infer_pre_start_date()
        self._process_fact_geo()
        self.ci = CausalImpact(self.data_ci, self.pre_period, self.post_period)
