"""

"""


from __future__ import absolute_import, division, print_function

import pandas as pd
import datetime
import os

from google.cloud import bigquery

from causaltest.bin.conf import GCLOUD_PROJECT_ID
from google.oauth2 import service_account

from causaltest.bin.secrets import SERVICE_ACCOUNT_CREDENTIALS
from causaltest.misc import str_to_date, date_to_str
from causaltest.bin.conf import QUERY_DIM_GEO, DATE_NAME, QUERY_FACT_TEST


def get_bigquery_client() -> bigquery.Client:
    # Big Query authentication: using standard google.oauth2 authentication
    # Adding the Drive scope in order to query an external table that lies on Google sheets.
    # TODO We should put a more restrictive scope (only read to spreadsheet, not all accesses to drive)
    credentials = service_account.Credentials.from_service_account_info(
        SERVICE_ACCOUNT_CREDENTIALS,
        scopes=["https://www.googleapis.com/auth/cloud-platform", "https://www.googleapis.com/auth/drive"])
    client = bigquery.Client(credentials=credentials, project=GCLOUD_PROJECT_ID)
    return client


def custom_read_gbq(client: bigquery.Client, query: str) -> pd.DataFrame:
    """
    :param client: Should be initialized to the proper project id
    :param query:
    """
    return client.query(query).to_dataframe()


def custom_write_gbq(client: bigquery.Client, target_dataset_name: str, target_table_name: str, df: pd.DataFrame, mode='erase') -> None:
    """
    Will write (and erase) the table DempF30days.temp_network_line_cross_od
    :param client: Should be initialized to the proper project id
    :param df: DataFrame to write to GBQ
    """
    full_id = f'{GCLOUD_PROJECT_ID}.{target_dataset_name}.{target_table_name}'

    # Delete the table
    if mode=='erase':
        client.delete_table(full_id, not_found_ok=True)

    # Write the new one
    dataset_ref = client.dataset(target_dataset_name)
    table_ref = dataset_ref.table(target_table_name)

    if mode == 'erase':
        client.load_table_from_dataframe(df, table_ref).result()
    elif mode == 'append':
        table = bigquery.table.Table(table_ref=table_ref)
        loaded_table = client.get_table(table=table)
        client.insert_rows_from_dataframe(table=loaded_table, dataframe=df)

    return None


def custom_read_gbq_saved_query(client: bigquery.Client, query_name: str, **kwargs) -> pd.DataFrame:
    """
    Read a query saved in the bin/queries folder
    """
    path = os.path.dirname(os.path.abspath(__file__)) # get root path
    file_path = os.path.join(path, 'bin', 'queries', query_name+'.sql')
    with open(file_path, 'r') as f:
        query = f.read()
    return custom_read_gbq(client=client, query=query.format(**kwargs))


def load_fact_test(test_id, query_name=QUERY_FACT_TEST): # TODO: should be brought back in class causaltest
    client = get_bigquery_client()
    query = 'SELECT * FROM `bbc-data-marketing-automation.incrementality.fact_test` WHERE test_id = "{test_id}"'
    return custom_read_gbq(client=client, query=query.format(test_id=test_id))


def load_fact_split(test_id):
    client = get_bigquery_client()
    query = 'SELECT * FROM `bbc-data-marketing-automation.incrementality.geo_split` WHERE test_id = "{test_id}"'
    return custom_read_gbq(client=client, query=query.format(test_id=test_id))
