"""
"""


from __future__ import absolute_import, division, print_function

import branca as branca
import folium as folium
import numpy as np
import pandas as pd
import shapely
from shapely.wkt import loads
import random
from typing import Union
import hashlib
import os

from causaltest.bin.conf import LAT_LONG, COLOR

GOOGLE_MAPS_API_KEY = os.environ['GOOGLE_MAPS_API_KEY']

class Mapper(object):
    """

    """
    def _generate_map(self):
        """
        Generate the map of the country with a color for control,
        another for treatment, and another for unattributed
        """
        network_map = folium.Map(
            location=LAT_LONG[self.country_code],#[47.078851, 2.398131],  # Somewhere in France define var and check if att country_code is ok
            tiles='https://mts1.google.com/vt/lyrs=m&x={x}&y={y}&z={z}',
            API_key=GOOGLE_MAPS_API_KEY,
            attr='Google Maps',
            max_zoom=18,
            zoom_start=6
        )

        for i, row in self.dim_geo.iterrows():
            if row.id in self.treatment_geo_id:
                color = COLOR['treatment']
                text = 'treatment'
            elif row.id in self.control_geo_id:
                color = COLOR['control']
                text = 'control'
            else:
                color = COLOR['excluded']
                text = 'excluded'

            shapely_object = loads(row.geography)

            popup = folium.Popup(
                html=f"Name: {row['id']} ({row['name']}).\n"
                f" Population: {row.population}.\n"
                f"Group: {text}")
            folium_object = self.shapely_to_folium(shapely_object, color=color).add_child(popup)
            folium_object.add_to(network_map)

        network_map.fit_bounds(network_map.get_bounds())

        self.network_map = network_map
        self.network_map_html = network_map._repr_html_() # TODO: should be taken elsewhere, would better fit inside the app logic rather then the test logic


    # Often, we have to swap lat and lon in geometric objects, to deal with various conventions
    # mostly civilized people use lon/lat; Google Maps uses lat/lon
    @staticmethod
    def swap_coords(list_coords, recursion_level=0):
        """
        Recursive function that returns the same list_coords with reversed latitude and longitude.
        list_coords: list of list of list (at any level of recursion) taken from a GeoJson coordinates object
        returns: the same object where we reverse the (lat, lon) to (lon, lat) or symmetrically.
        """
        #print("Entering swap_coords: recursion level=", recursion_level)
        #print(list_coords)

        array_like = [tuple, list, np.array]
        if type(list_coords) in array_like: #has a len() method
            if len(list_coords)==1:
                #print("length 1")
                return (swap_coords(list_coords[0], recursion_level=recursion_level+1),)

            elif len(list_coords)==2:
                # print("length 2")
                # print("types:", type(list_coords[0]), type(list_coords[1]))
                if type(list_coords[0]) == float and type(list_coords[1]) == float:
                    # print("tuple of coordinates")
                    # it is a tuple of coordinates
                    return (list_coords[1], list_coords[0])
                else:
                    # print("not coordinates: calling separately")
                    return (swap_coords(list_coords[0], recursion_level=recursion_level+1), swap_coords(list_coords[1], recursion_level=recursion_level+1))

            elif len(list_coords)>=3:
                return [swap_coords(list_coords[i], recursion_level=recursion_level+1) for i in range(len(list_coords))]
            else:
                raise ValueError("please check")

    @staticmethod
    def shapely_to_folium(shapely_object, swap_lat_lon: bool = False, weight: float = 4, color: str = None):
        geojson = shapely.geometry.mapping(shapely_object)
        if swap_lat_lon:
            geojson = shapely.geometry.mapping(shapely_object)
            swapped_coordinates = self.swap_coords(geojson['coordinates'])
            geojson['coordinates'] = swapped_coordinates
        if color is None:
            color = self.generate_random_hexa_color()

        return folium.GeoJson(
            geojson,
            style_function=lambda feature: {
              'color': color,
              'weight': weight
            },
            highlight_function=
            lambda feature:
                {
                    'color': color,
                    'weight': weight,
                    'opacity': 0.5
                }
                if weight > 5
                else {
                    'color': color,
                    'weight': weight,
                    'dashArray': '5 5',
                    'lineCap': 'square',
                    'lineJoin': 'square',
                }
        )

    @staticmethod
    def generate_random_hexa_color(seed: Union[int, str] = None):
        """
        Will generate a string representing a 6-digits hexadecimal (with # at beginning) to generate a random color
        :param seed: If seed is specified, we initialize the seed. Can be either a string or an int
        We want that for any integer (say, 3), the return of the function be the same than the
        coerced equivalent string ('3')
        :return:
        """
        if type(seed) == str:
            int_seed = int(hashlib.sha256(seed.encode('utf-8')).hexdigest(), 16) % 10**8
            random.seed(int_seed)
        elif type(seed) == int:
            # 3 and '3' will return the same seed
            int_seed = int(hashlib.sha256(str(seed).encode('utf-8')).hexdigest(), 16) % 10 ** 8
            random.seed(int_seed)
        return "#"+''.join([random.choice('0123456789ABCDEF') for j in range(6)])
